#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/stat.h>
#include <unistd.h>

#ifndef D20_LIB_DIR
#define D20_LIB_DIR "/usr/lib/d20"
#endif

static void usage(void)
{
    puts("d20 [command] [parameters]");
}

int main(int ac, char **av)
{
    char const *command = NULL;
    char *file = NULL;
    char **args = NULL;
    struct stat s;
    int i = 0, ret = 0;

    if (ac <= 1) {
        usage();
        exit(0);
    }

    if (av[1][0] == '-') {
        /* parameter given, assume default command `dice`
         */
        command = "dice";
    } else {
        command = av[1];
        ++av;
        --ac;
    }

    asprintf(&file, "%s/d20-%s", D20_LIB_DIR, command);

    if (stat(file, &s) != 0) {
        fprintf(stderr, "failed to run command: %s\n", strerror(errno));
        exit(3);
    }

    args = calloc(ac + 1, sizeof(char*));
    if (args == NULL) {
        exit(3);
    }

    asprintf(&args[0], "d20-%s", command);
    for (i = 1; i < ac; i++) {
        args[i] = strdup(av[i]);
    }

    ret = execv(file, args);
    if (WIFEXITED(ret) && WEXITSTATUS(ret) != 0) {
        return WEXITSTATUS(ret);
    } else if (!WIFEXITED(ret)) {
        return 127;
    }

    return 0;
}
